#include <States/SelectState/SelectState.h>
#include <States/ChangeStateCommand.h>
#include <BasicAbstractions/Font.h>

void Button::set(const sf::Vector2f pos, const sf::Vector2f size, const std::string& text, const size_t font_size, std::unique_ptr<ISelectCommand> ptr_command) {
    m_rectangle.setSize(size);
    m_rectangle.setPosition(pos);
    m_rectangle.setFillColor(config::BUTTON_COLOR_FILL);
    m_rectangle.setOutlineThickness(config::BUTTON_FRAME_THICKNESS);
    m_rectangle.setOutlineColor(config::BUTTON_COLOR_FRAME);
    m_text.setFont(MyFont::Instance());
    m_text.setCharacterSize(font_size);
    m_text.setFillColor(config::BUTTON_COLOR_TEXT);
    m_text.setString(text);
    const sf::FloatRect text_bounds = m_text.getLocalBounds();
    const sf::Vector2f button_pos = m_rectangle.getPosition();
    const sf::Vector2f button_size = m_rectangle.getSize();
    m_text.setPosition(
        round(button_pos.x + (button_size.x - text_bounds.width) / 2 - text_bounds.left),
        round(button_pos.y + (button_size.y - text_bounds.height) / 2 - text_bounds.top)
    );
    m_ptr_command = std::move(ptr_command);
}

void Button::select() {
    m_rectangle.setFillColor(config::BUTTON_COLOR_SELECTION);
}

void Button::unselect() {
    m_rectangle.setFillColor(config::BUTTON_COLOR_FILL);
}

bool Button::is_position_in(const sf::Vector2f pos) const noexcept {
    return pos.x >= m_rectangle.getPosition().x && pos.x <= (m_rectangle.getPosition().x + m_rectangle.getSize().x)
        && pos.y >= m_rectangle.getPosition().y && pos.y <= (m_rectangle.getPosition().y + m_rectangle.getSize().y);
}

void Button::push() const {
    m_ptr_command->execute();
}

void Button::draw_into(sf::RenderWindow& window) const {
    window.draw(m_rectangle);
    window.draw(m_text);
}

Menu::Menu(IStateManager& state_manager) {
    const float pos_left = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.width) - config::BUTTON_SIZE.x) / 2;
    const float pos_diff = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.height) - config::BUTTON_SIZE.y * 4) / 10;
    const float pos_top = (static_cast<float>(config::SELECT_LEVEL_VIDEO_MODE.height) - config::BUTTON_SIZE.y*4 - pos_diff*3) / 2;
    m_buttons[0].set(sf::Vector2f{pos_left, pos_top}, config::BUTTON_SIZE, config::BUTTON_TEXT_EASY,
        config::BUTTON_FONT_SIZE, std::make_unique<ExitCommand>(state_manager));
    m_buttons[1].set(sf::Vector2f{pos_left, pos_top + config::BUTTON_SIZE.y + pos_diff}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_MEDIUM, config::BUTTON_FONT_SIZE, std::make_unique<GameCommand>(state_manager, std::make_unique<GameBuilderDirector>()));
    m_buttons[2].set(sf::Vector2f{pos_left, pos_top + 2*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_HARD, config::BUTTON_FONT_SIZE, std::make_unique<GameCommand>(state_manager, std::make_unique<GameBuilderDirector>()));
    m_buttons[3].set(sf::Vector2f{pos_left, pos_top + 3*(config::BUTTON_SIZE.y + pos_diff)}, config::BUTTON_SIZE,
     config::BUTTON_TEXT_EXIT, config::BUTTON_FONT_SIZE, std::make_unique<GameCommand>(state_manager, std::make_unique<GameBuilderDirector>()));
}

void Menu::process_mouse(const sf::Vector2f pos, const bool is_pressed) {
    for (auto& button : m_buttons) {
        if (!button.is_position_in(pos)) {
            button.unselect();
            continue;
        }
        if (!is_pressed) {
            button.select();
            continue;
        }
        button.push();
    }
}

void Menu::draw_into(sf::RenderWindow& window) const {
    for (auto& button : m_buttons) {
        button.draw_into(window);
    }
}

void SelectState::event_handling() {
    sf::Event event{};
    while (m_window.pollEvent(event)) {
        if (event.type == sf::Event::Closed) {
            m_state_manager.set_next_state(std::make_unique<ExitState>(m_state_manager));
            m_window.close();
        }
        if (event.type == sf::Event::Resized) {
            sf::View view = m_window.getView();
            view.setSize(event.size.width, event.size.height);
            m_window.setView(view);
        }
    }
}

void SelectState::update() {
    m_menu.process_mouse(m_window.mapPixelToCoords(sf::Mouse::getPosition(m_window)),
        sf::Mouse::isButtonPressed(sf::Mouse::Left));
}

void SelectState::render() {
    m_window.clear(config::SELECT_LEVEL_BACKGROUND_COLOR);
    m_menu.draw_into(m_window);
    m_window.display();
}

bool SelectState::do_step() {
    event_handling();
    update();
    render();
    return true;
}